<%@Page Language="C#" %>
<%@Import namespace="Sitecore.Data" %>
<%@Import namespace="Sitecore.Data.Items" %>
<%@Import namespace="Sitecore.SecurityModel" %>
<%@Import namespace="Sitecore.Data.Events" %>
<%@Import namespace="System.IO" %>
<%@Import namespace="System.Reflection" %>
<%@Import namespace="System.Threading" %>

<html>
<head>
    <style>
        .version {
            font-weight: bold
        }
        .warning {
           color: red
        }
        .log {
            display: inline-block;
            white-space: nowrap;
        }
        .refresh {
            padding: 5px;
            margin-bottom: 5px;
        }
        .running {
            background: lightcoral;
        }
        .finished {
            background: lightgreen;
        }
    </style>

<script runat="server">
    public const int REFRESH_MS = 1000;
    public static readonly string[] DB_PATHS_CORE = new string[] 
    {
        //{B69EB822-9BFC-4226-ACD8-18E05708E66E}
        "/sitecore/content/Applications/Content Editor/Editors/Data Exchange Framework", 
        //{3F3E933E-DF5C-4347-98E2-C1B7A80E37EE}
        "/sitecore/content/Applications/Content Editor/Ribbons/Contextual Ribbons/Data Exchange", 
        //{1BAB6B5E-C1C6-41E4-AA50-BC45ADB9ABFB}
        "/sitecore/content/Applications/Content Editor/Ribbons/Ribbons/Default/Data Exchange SDK", 
        //{2D88B95A-AA0A-4542-8061-2A8AE4953E61}
        "/sitecore/content/Applications/Content Editor/Ribbons/Strips/Data Exchange SDK", 
        //{ECB2614C-4F08-4087-8924-2C8224DE7D7B}
        "/sitecore/content/Applications/Data Exchange Framework SDK", 
    };
    public static readonly string[] DB_PATHS_MASTER = new string[] 
    {
        "/sitecore/system/Data Exchange",
        "/sitecore/system/Settings/Data Exchange",
        "/sitecore/system/Settings/Rules/Definitions/Macros/Data Exchange Framework",
        "/sitecore/system/Tasks/Commands/Data Exchange",
        "/sitecore/templates/Branches/Data Exchange",
        "/sitecore/templates/Data Exchange",
    };
    public static readonly string[] FOLDER_PATHS = new string[]
    {
        @"App_Config\Include\DynamicsCrm",
        @"App_Config\Include\Salesforce",
        @"sitecore\shell\Applications\Content Manager\Editors\Queue",
        @"sitecore\shell\Applications\Dialogs\Data Exchange Framework SDK",
        @"temp\Data Exchange Framework",
        @"temp\Dynamics CRM",
        @"temp\Salesforce",
    };
    public static readonly string[] FILE_PATHS = new string[]
    {
        @"App_Config\Include\Sitecore.DataExchange.*",
        @"bin\Sitecore.DataExchange.dll",
        @"bin\Sitecore.DataExchange.*.dll",
        @"bin\Sitecore.Connect.Crm.dll",
        @"bin\Sitecore.Connect.Crm.*.dll",
        @"bin\Sitecore.Analytics.DynamicsCrm.dll",
        @"bin\Sitecore.Analytics.Salesforce.dll",
    };

    public int RecycleItems(Database database, IEnumerable<Item> items)
    {
        var count = 0;
        using (new EventDisabler())
        {
            using (new SecurityDisabler())
            {
                using (new StatisticDisabler())
                {
                    foreach (var item in items)
                    {
                        var parameters = new string[] 
                        {
                            "database: " + database.Name, 
                            "id: " + item.ID, 
                            "descendant count: " + item.Axes.GetDescendants().Length,
                            "path: " + item.Paths.FullPath
                        };
                        if (! this.IsDebugMode)
                        {
                            Log("Starting to delete item...", parameters);   
                            item.Recycle();
                            Log("Finished deleting item.", parameters);   
                        }
                        else
                        {
                            Log("Item not deleted in debug mode.", parameters);
                        }
                        count++;
                    }
                }
            }
        }
        return count;
    }
    public int RecycleFolders(IEnumerable<string> paths)
    {
        var count = 0;
        var basePath = System.AppDomain.CurrentDomain.BaseDirectory;
        foreach (var path in paths)
        {
            var path2 = Path.Combine(basePath, path);
            if (Directory.Exists(path2))
            {
                var msg = "Directory deleted.";
                if (! this.IsDebugMode)
                {
                    Directory.Delete(path2, true);
                }
                else
                {
                    msg = "Directory not deleted in debug mode.";
                }
                Log(msg, "path: " + path2);
                count++;
            }
        }
        return count;
    }
    public int RecycleFiles(IEnumerable<string> paths)
    {
        var count = 0;
        var basePath = System.AppDomain.CurrentDomain.BaseDirectory;
        foreach (var path in paths)
        {
            var path2 = Path.Combine(basePath, path);
            if (File.Exists(path))
            {
                if (! this.IsDebugMode)
                {
                    File.Delete(path);
                }
                Log("File deleted.", "path: " + path);
                count++;
            }
        }
        return count;
    }
    public StringBuilder Buffer { get; set; }
    public void Log(string msg, params string[] values)
    {
        var builder = new StringBuilder();
        builder.Append(DateTime.Now.ToString("yyyy-MM-dd hh:mm:ss"));
        builder.Append("  ");
        if (this.IsDebugMode)
        {
            builder.Append("DEBUG  ");
        }
        builder.Append(msg);
        if (values != null && values.Length > 0)
        {
            builder.Append(" (" + string.Join(", ", values) + ")");
        }
        Buffer.Append(builder.ToString());
        Buffer.Append("<br/>");
        WriteStringBuilderForLogging();
    }
    public IEnumerable<Item> GetItemsByPaths(Database database, IEnumerable<string> paths)
    {
        var items = new List<Item>();
        using (new SecurityDisabler())
        {
            foreach (var path in paths)
            {
                var item = database.GetItem(path);
                if (item != null)
                {
                    items.Add(item);
                }
            }
        }
        return items;
    }
    public IEnumerable<Item> GetItemsByName(Database database, string path, string name, bool fuzzy = false)
    {
        var items = new List<Item>();
        var parent = database.GetItem(path);
        if (parent != null && parent.HasChildren)
        {
            foreach (Item child in parent.Children)
            {
                var addChild = false;
                if (!fuzzy)
                {
                    addChild = (child.Name == name);
                }
                else
                {
                    addChild = (child.Name.StartsWith(name));
                }
                if (addChild)
                {
                    items.Add(child);
                }
            }
        }
        return items;
    }
    public IEnumerable<string> GetFilePathsByName(string path)
    {
        var basePath = System.AppDomain.CurrentDomain.BaseDirectory;
        path = Path.Combine(basePath, path);
        var paths = new List<string>();
        var directoryName = Path.GetDirectoryName(path);
        var fileName = Path.GetFileName(path);
        if (!Directory.Exists(directoryName))
        {
            return Enumerable.Empty<string>();
        }
        foreach (var filePath in Directory.GetFiles(directoryName, fileName))
        {
            if (File.Exists(filePath))
            {
                paths.Add(filePath);
            }
        }
        return paths;
    }
    protected Assembly GetAssemblyByName(string name)
    {
        return AppDomain.CurrentDomain.GetAssemblies().FirstOrDefault(a => a.GetName().Name == name);
    }
    protected bool IsDynamicsInstalled()
    {
        return (GetAssemblyByName("Sitecore.Connect.Crm.DynamicsCrm") != null);
    }
    protected bool IsSalesforceInstalled()
    {
        return (GetAssemblyByName("Sitecore.Connect.Crm.Salesforce") != null);
    }
    public bool IsDebugMode 
    {
        get 
        {
            return chkDebugMode.Checked;
        }
    }
    void StartUninstall()
    {
        Log("Starting the uninstall process...");
        var core = Database.GetDatabase("core");
        var coreItems = new List<Item>();
        coreItems.AddRange(GetItemsByPaths(core, DB_PATHS_CORE));
        Log("Starting to delete items from the core database...");
        var countCore = RecycleItems(core, coreItems);
        Log("Finished deleting " + countCore + " item(s) from the core database.");

        var master = Database.GetDatabase("master");
        var masterItems = new List<Item>();
        masterItems.AddRange(GetItemsByPaths(master, DB_PATHS_MASTER));
        masterItems.AddRange(GetItemsByName(master, "/sitecore/system/Settings/Rules/Insert Options/Rules", "Data Exchange", true));
        masterItems.AddRange(GetItemsByName(master, "/sitecore/system/Settings/Rules/Insert Options", "Data Exchange", true));
        Log("Starting to delete items from the master database...");
        var countMaster = RecycleItems(master, masterItems);
        Log("Finished deleting " + countMaster + " item(s) from the master database.");
        
        using (new SecurityDisabler())
        {
            if (! this.IsDebugMode)
            {
                if (countCore > 0)
                {
                    Log("Rebuilding link database for core database...");
                    Sitecore.Globals.LinkDatabase.Rebuild(core);
                    Log("Finished rebuilding link database for core database.");
                }
            }
            else 
            {
                Log("Link database for core database will not be rebuilt in debug mode.");
            }
            if (! this.IsDebugMode)
            {
                if (countMaster > 0)
                {
                    Log("Rebuilding link database for master database...");
                    Sitecore.Globals.LinkDatabase.Rebuild(master);
                    Log("Finished rebuilding link database for master database.");
                }
            }
            else
            {
                Log("Link database for master database will not be rebuilt in debug mode.");
            }
            if (! this.IsDebugMode)
            {
                if (countCore > 0 || countMaster > 0)
                {
                    Log("Clearing caches...");
                    Sitecore.Caching.CacheManager.ClearAllCaches();
                    Log("Finished clearing caches.");
                }
            }
            else
            {
                Log("Caches will not be cleared in debug mode.");
            }
        }
        
        Log("Starting to delete folders (refresh might stop while app pool restarts)...");
        Thread.Sleep(REFRESH_MS);
        var count = RecycleFolders(FOLDER_PATHS);
        Log("Finished deleting " + count + " folder(s).");
        var filesToRecycle = new List<string>();
        foreach (var file in FILE_PATHS)
        {
            filesToRecycle.AddRange(GetFilePathsByName(file));
        }
        Log("Starting to delete files (refresh might stop while app pool restarts)...");
        Thread.Sleep(REFRESH_MS);
        count = RecycleFiles(filesToRecycle);
        Log("Finished deleting " + count + " file(s).");
        Log("Finished the uninstall process.");
    }
    protected void UpdateTimer_Tick(object sender, EventArgs e)
    {
        if (!panelRefresh.Visible)
        {
            panelRefresh.Visible = true;
        }
        string msg = null;
        string css = null;
        if (IsRunning) 
        {
            css = "running";
            msg = "Last refresh";
        }
        else 
        {
            css = "finished";
            css = "finished";
            msg = "Finished running";
            updateTimer.Enabled = false;
            btnUninstall.Enabled = true;
            chkDebugMode.Enabled = true;
        }
        panelRefresh.CssClass = "refresh " + css;
        lblLastRefresh.Text = msg + " at <code>" + DateTime.Now.ToString("yyyy-MM-dd hh:mm:ss") + "</code>";
        lblOutput.Text = Buffer.ToString();
    }
    public static bool IsRunning = false;
    public static Thread UninstallThread = null;
    protected void Uninstall_OnClick(object sender, EventArgs e)
    {
        btnUninstall.Enabled = false;
        chkDebugMode.Enabled = false;
        Buffer.Clear();
        updateTimer.Enabled = true;
        UninstallThread = new Thread(() =>
        {
            IsRunning = true;
            StartUninstall();
            IsRunning = false;
        });
        if (File.Exists(this.LogFilePath))
        {
            File.Delete(this.LogFilePath);
        }
        UninstallThread.Start();
    }
    protected void chkDebugMode_OnCheckedChanged(object sender, EventArgs e)
    {
    }
    public string LogFilePath
    {
        get 
        {
            return Path.Combine(Path.GetTempPath(), "defuninstall.log");
        }
    }
    public void LoadStringBuilderForLogging()
    {
        this.Buffer = new StringBuilder();
        if (File.Exists(this.LogFilePath))
        {
            using (var reader = File.OpenText(this.LogFilePath))
            {
                string s = String.Empty;
                while ((s = reader.ReadLine()) != null)
                {
                    this.Buffer.AppendLine(s);
                }
            }
        }
    }
    public void WriteStringBuilderForLogging()
    {
        File.WriteAllText(this.LogFilePath, this.Buffer.ToString());
    }
    protected void Page_Init(object sender, EventArgs e)
    {
        //
        //load buffer from file
        LoadStringBuilderForLogging();
        //
        //
        if (!Page.IsPostBack)
        {
            updateTimer.Interval = REFRESH_MS;
            panelCanUninstall.Visible = false;
            var asm = GetAssemblyByName("Sitecore.DataExchange");
            if (asm == null)
            {
                lblVersion.Text = "None";
                lblMessage.Text = "DEF is not installed on this server. You can still run the uninstaller.";
                panelWarning.Visible = true;
                panelCanUninstall.Visible = true;
            }
            else 
            {
                var builder = new StringBuilder();
                if (IsDynamicsInstalled())
                {
                    builder.Append(@"<li>Dynamics CRM Connect</li>");
                }
                if (IsSalesforceInstalled())
                {
                    builder.Append(@"<li>Salesforce Connect</li>");
                }
                if (builder.Length == 0) 
                {
                    builder.Append("None");
                }
                lblTools.Text = builder.ToString();
                var version = asm.GetName().Version;
                lblVersion.Text = version.ToString();
                string op = null;
                if (version.Major > 1)
                {
                    op = @"<span style='font-weight:bold; color:red'>cannot</span>";
                }
                else
                {
                    op = @"<span style='font-weight:bold; color:green'>can</span>";
                    panelWarning.Visible = true;
                    panelCanUninstall.Visible = true;
                }
                lblMessage.Text = "This script " + op + " be used to uninstall the version of DEF installed on this server.";
            }
        }
    }
</script>

<title>Data Exchange Framework Uninstaller</title>

</head>
<body>
    <h3>Data Exchange Framework Uninstaller</h3>
    <div style="width:600px">
        <form runat="server">
            <asp:ScriptManager ID="ScriptManager1" runat="server" />
            <table style="background:whitesmoke">
                <tr>
                    <td width="25%">Installed version:</td>
                    <td class="version" width="100%"><code><asp:Label ID="lblVersion" runat="server" /></code></td>
                </tr>
                <tr>
                    <td width="25%">Supported versions:</td>
                    <td class="version" width="100%"><code>1.x</code></td>
                </tr>
                <tr>
                    <td style="vertical-align:top;" width="25%">Detected tools:</td>
                    <td class="version" width="100%"><code><asp:Label ID="lblTools" runat="server" /></code></td>
                </tr>
                <tr>
                    <td colspan="2"><asp:Label ID="lblMessage" CssClass="message" runat="server" /></td>
                </tr>
            </table>
            <asp:Panel ID="panelCanUninstall" runat="server">
                <asp:Panel ID="panelDynamicsDetected" runat="server" Visible=false>
                    <p class="warning">
                        <b>WARNING:</b> Dynamics CRM Connect was detected. 
                        This product will be uninstalled.
                    </p>
                </asp:Panel>
                <asp:Panel ID="panelSalesforceDetected" runat="server" Visible=false>
                    <p class="warning">
                        <b>WARNING:</b> Salesforce Connect was detected. 
                        This product will be uninstalled.
                    </p>
                </asp:Panel>
                <asp:Panel ID="panelInstructions" runat="server">
                    <p>
                        <b>INSTRUCTIONS</b><br/>
                        This uninstaller will remove Data Exchange Framework and 
                        any tools listed above.
                        <br/><br/>
                        Running the uninstaller in <i>debug</i> mode will show 
                        you the changes that will be made without actually
                        making the changes. You can use this to understand
                        how the uninstaller will affect your server.
                    </p>
                </asp:Panel>
                <asp:Panel ID="panelWarning" runat="server">
                    <p class="warning">
                        <b>WARNING</b><br/>
                        Backup your Sitecore server before using this tool.<br/><br/>
                        This tool will remove components from 
                        your Sitecore server. While it moves Sitecore items 
                        into the <i>Sitecore Recycle Bin</i> and files into 
                        the <i>Windows Recycle Bin</i> (for the app pool account), 
                        you must use this tool carefully.
                    </p>
                </asp:Panel>
                <asp:UpdatePanel runat="server" id="panelUninstallStatus">
                    <ContentTemplate>
                        <p>
                            <asp:Button ID="btnUninstall" Text="Uninstall" runat="server" OnClick="Uninstall_OnClick" />
                            <asp:Checkbox ID="chkDebugMode" 
                                          Text="Debug mode" 
                                          Checked=true 
                                          runat="server" 
                                          AutoPostBack="true" 
                                          OnCheckedChanged="chkDebugMode_OnCheckedChanged" />
                        </p>
                        <asp:Timer runat="server" ID="updateTimer" OnTick="UpdateTimer_Tick" Enabled=false />
                        <asp:Panel runat="server" ID="panelRefresh" CssClass="refresh" Visible=false>
                            <asp:Label id="lblLastRefresh" runat="server" />
                        </asp:Panel>
                        <div class="log">
                            <code>
                                <asp:Label id="lblOutput" runat="server" Width="100%" />
                            </code>
                        <div>
                    </ContentTemplate>
                </asp:UpdatePanel>
            </asp:Panel>
        </form>
    </div>
</body>
</html>